require 'yaml'
require 'active_support/all'
require 'date'
require_relative '../service/service_response_data'
require_relative 'dellininfo/base_api'
require_relative 'dellininfo/client'
require_relative 'dellininfo/reference'
require_relative 'dellininfo/order_collection'
require_relative 'dellininfo/reference'
require_relative 'dellininfo/order_status'
module DellinInfo
  module API

  PATH = 'path to config.yml'


    # Приложение можно сконфигурировать, для конфикурации необходимо указать в константе DellinInfo::API::PATH путь до файла yml
    # где ключ - название класса в snakecase, а значение - пара (параметр - значение)
    # Например Client:
    #                 app_key: 1234
    # Все основные настройки хранятся в отдельном файле, данный метод рализует конфигурацию всех классов модуля
    def self.configure!
      YAML.load_file(PATH).each_pair do |config_class, params|
        params.each_pair do |param, value|
          "DellinInfo::API::#{config_class.camelize}".constantize.config[param] = value
        end
      end
    end


    # Данный метод получает ссылку на csv с данными справочников Деловых линиях
    # http://dev.dellin.ru/api/public/tables/
    # и хэш, по которому можно отследить актуальность данных
    # возвращается объект класса DellinInfo::API::Reference
    def self.reference(type)
      type = type.to_sym
      url_type = Reference::TYPE[type]
      data = client_request(url_type)

      Reference.new(data)
    end


    # метод поиска накладных по неполным данным
    # параметры метода есть в readme
    # метод возвращает коллецию заказов, объект класса DellinInfo::API::OrderCollection
    def self.find_orders(params = {})
      if params[:date_start].nil?
        set_time!
        params[:date_start] = @start_date
        params[:date_end] = @end_day
      end
      data = client_request(OrderCollection, params)
      OrderCollection.new data
      
    end


    # метод проверяющий статусы накладной по номеру
    def self.status_oder(id_oder)
      data = client_request(OrderStatus, docid: id_oder)
      DellinInfo::API::OrderStatus.new data
      
    end

    # метод для запроса на ресурс "dellin.ru", принимает класс (какую сущность мы хотим получить)
    # и параметры - тело запроса (data)
    def self.client_request(dellin_class, params = {})
      client = Client.new()
      data = client.request(dellin_class.config['partial_url'], params).body
      begin
      JSON.parse data
      rescue JSON::ParserError => e
        return {'errormsg' => 'Похоже сервер Деловых Линий чем-то занят и не может сейчас ответить, попробуйте повторить запрос позже', 'json_err' => e.message}
      end
      
    end

    # метод для установки время по умолчанию (неделя)
    def self.set_time!
      @start_date  = 6.day.ago.strftime('%Y-%m-%d')
      @end_day     = Time.now.strftime('%Y-%m-%d')
    end


  end
end
